package jaredbgreat.dldungeons.builder;


/*This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/	


import jaredbgreat.dldungeons.planner.Dungeon;

import java.util.Random;

import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.tileentity.TileEntityMobSpawner;
import net.minecraft.world.World;
import net.minecraft.world.chunk.IChunkProvider;

public class Builder {
	private static boolean debugPole;
	
	
	public static void placeBlock(World world, int x, int y, int z, int block) {
		// This wrapper is a protection against possible changes in block representation,
		// e.g., abandoning the ID system, allowing any needed changes to be made here
		// instead of elsewhere. 
		world.setBlock(x, y, z, block);
	}
	
	
	public static void placeBlock(World world, int x, int y, int z, int block, int a, int b) {
		// This wrapper is a protection against possible changes in block representation,
		// e.g., abandoning the ID system, allowing any needed changes to be made here
		// instead of elsewhere. 
		world.setBlock(x, y, z, block, a, b);
	}
	
	
	public static void deleteBlock(World world, int x, int y, int z) {
		// This wrapper is a protection against possible changes in block representation,
		// e.g., abandoning the ID system, allowing any needed changes to be made here
		// instead of elsewere. 
		// world.setBlock(x, y, z, 0);
		//if(isProtectedBlock(world, x, y, z)) return;
		world.setBlockToAir(x, y, z);
	}
	
	
	public static void placeSpawner(World world, int x, int y, int z, String mob) {
		placeBlock(world, x, y, z, Block.mobSpawner.blockID);
		TileEntityMobSpawner spawner = (TileEntityMobSpawner)world.getBlockTileEntity(x, y, z);
		spawner.func_98049_a().setMobID(mob);
		//world.setBlockMetadataWithNotify(x,y,z, 15, 7);
	}
	
	
	public static boolean isGroundBlock(World world, int x, int y, int z) {
		Material mat = world.getBlockMaterial(x, y, z);
		return (mat == Material.grass) || (mat == Material.ground) || (mat == Material.sand) 
				|| (mat == Material.rock) || (mat == Material.coral) || (mat == Material.craftedSnow) 
				|| (mat == Material.clay);
	}
	
	
	public static boolean isProtectedBlock(World world, int x, int y, int z) {
		int block = world.getBlockId(x, y, z);
		return (Block.chest.isAssociatedBlockID(block) || Block.mobSpawner.isAssociatedBlockID(block) 
				|| Block.endPortalFrame.isAssociatedBlockID(block) || Block.endPortal.isAssociatedBlockID(block));
	}
	

	public static void placeDungeon(Random random, int chunkX, int chunkZ, World world,
						IChunkProvider chunkGenerator, IChunkProvider chunkProvider) throws Throwable {	
		if(world.isRemote) return; // Do not perform world-gen on the client!
		//Debug Code
		//System.out.println("Running Builder.placeDungeon; generating dungeon");
		Dungeon dungeon = new Dungeon(random, world.getBiomeGenForCoords((chunkX * 16), (chunkZ * 16)), 
									  world, chunkX, chunkZ);

		System.out.println("[DLDUNGONS] Running Builder.buildDungeon; building dungeon");
		buildDungeon(dungeon);
		System.out.println("[DLDUNGONS] Dungeon should be built now!");
		if(debugPole) debuggingPole(world, chunkX, chunkZ, dungeon);
		dungeon.finalize();
		dungeon = null;
	}
	
	
	public static void buildDungeon(Dungeon dungeon /*TODO: Parameters*/) {
		//System.out.println("[DLDUNGONS] Inside Builder.placeDungeon; building dungeon");
		dungeon.map.build(dungeon);
	}
	
	
	public static void debuggingPole(World world, int chunkX, int chunkZ, Dungeon dungeon) {
		//Pointless stand-in for generation testing, and to help find dungeons later
		int x = (chunkX * 16) + 8;
		int z = (chunkZ * 16) + 8;
		for(int y = 16; y <= 241; y++) placeBlock(world, x, y, z, Block.blockNetherQuartz.blockID);
		for(int i = -dungeon.size.radius; i <= dungeon.size.radius; i++) {
			placeBlock(world, x - dungeon.size.radius, 80, z + i, Block.blockLapis.blockID);
			placeBlock(world, x + dungeon.size.radius, 80, z + i, Block.blockLapis.blockID);
			placeBlock(world, x + i, 80, z - dungeon.size.radius, Block.blockLapis.blockID);
			placeBlock(world, x + i, 80, z + dungeon.size.radius, Block.blockLapis.blockID);
		}		
	}
	
	
	public static void setDebugPole(boolean val) {
		debugPole = val;
	}
}
